// ------------------------------------------------------------------------------
//  Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
// ------------------------------------------------------------------------------

// **NOTE** This file was generated by a tool and any changes will be overwritten.
// <auto-generated/>

// Template Source: EntityType.cs.tt

namespace Microsoft.Graph
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Text.Json.Serialization;

    /// <summary>
    /// The type Content Type.
    /// </summary>
    public partial class ContentType : Entity
    {
    
        /// <summary>
        /// Gets or sets associated hubs urls.
        /// List of canonical URLs for hub sites with which this content type is associated to. This will contain all hubsites where this content type is queued to be enforced or is already enforced. Enforcing a content type means that the content type will be applied to the lists in the enforced sites.
        /// </summary>
        [JsonPropertyName("associatedHubsUrls")]
        public IEnumerable<string> AssociatedHubsUrls { get; set; }
    
        /// <summary>
        /// Gets or sets description.
        /// The descriptive text for the item.
        /// </summary>
        [JsonPropertyName("description")]
        public string Description { get; set; }
    
        /// <summary>
        /// Gets or sets document set.
        /// Document Set metadata.
        /// </summary>
        [JsonPropertyName("documentSet")]
        public DocumentSet DocumentSet { get; set; }
    
        /// <summary>
        /// Gets or sets document template.
        /// Document template metadata. To make sure that documents have consistent content across a site and its subsites, you can associate a Word, Excel, or PowerPoint template with a site content type.
        /// </summary>
        [JsonPropertyName("documentTemplate")]
        public DocumentSetContent DocumentTemplate { get; set; }
    
        /// <summary>
        /// Gets or sets group.
        /// The name of the group this content type belongs to. Helps organize related content types.
        /// </summary>
        [JsonPropertyName("group")]
        public string Group { get; set; }
    
        /// <summary>
        /// Gets or sets hidden.
        /// Indicates whether the content type is hidden in the list's 'New' menu.
        /// </summary>
        [JsonPropertyName("hidden")]
        public bool? Hidden { get; set; }
    
        /// <summary>
        /// Gets or sets inherited from.
        /// If this content type is inherited from another scope (like a site), provides a reference to the item where the content type is defined.
        /// </summary>
        [JsonPropertyName("inheritedFrom")]
        public ItemReference InheritedFrom { get; set; }
    
        /// <summary>
        /// Gets or sets is built in.
        /// Specifies if a content type is a built-in content type.
        /// </summary>
        [JsonPropertyName("isBuiltIn")]
        public bool? IsBuiltIn { get; set; }
    
        /// <summary>
        /// Gets or sets name.
        /// The name of the content type.
        /// </summary>
        [JsonPropertyName("name")]
        public string Name { get; set; }
    
        /// <summary>
        /// Gets or sets order.
        /// Specifies the order in which the content type appears in the selection UI.
        /// </summary>
        [JsonPropertyName("order")]
        public ContentTypeOrder Order { get; set; }
    
        /// <summary>
        /// Gets or sets parent id.
        /// The unique identifier of the content type.
        /// </summary>
        [JsonPropertyName("parentId")]
        public string ParentId { get; set; }
    
        /// <summary>
        /// Gets or sets propagate changes.
        /// If true, any changes made to the content type will be pushed to inherited content types and lists that implement the content type.
        /// </summary>
        [JsonPropertyName("propagateChanges")]
        public bool? PropagateChanges { get; set; }
    
        /// <summary>
        /// Gets or sets read only.
        /// If true, the content type cannot be modified unless this value is first set to false.
        /// </summary>
        [JsonPropertyName("readOnly")]
        public bool? ReadOnly { get; set; }
    
        /// <summary>
        /// Gets or sets sealed.
        /// If true, the content type cannot be modified by users or through push-down operations. Only site collection administrators can seal or unseal content types.
        /// </summary>
        [JsonPropertyName("sealed")]
        public bool? Sealed { get; set; }
    
        /// <summary>
        /// Gets or sets base.
        /// Parent contentType from which this content type is derived.
        /// </summary>
        [JsonPropertyName("base")]
        public ContentType Base { get; set; }
    
        /// <summary>
        /// Gets or sets base types.
        /// The collection of content types that are ancestors of this content type.
        /// </summary>
        [JsonPropertyName("baseTypes")]
        public IContentTypeBaseTypesCollectionWithReferencesPage BaseTypes { get; set; }

        /// <summary>
        /// Gets or sets baseTypesNextLink.
        /// </summary>
        [JsonPropertyName("baseTypes@odata.nextLink")]
        [JsonConverter(typeof(NextLinkConverter))]
        public string BaseTypesNextLink { get; set; }
    
        /// <summary>
        /// Gets or sets column links.
        /// The collection of columns that are required by this content type
        /// </summary>
        [JsonPropertyName("columnLinks")]
        public IContentTypeColumnLinksCollectionPage ColumnLinks { get; set; }

        /// <summary>
        /// Gets or sets columnLinksNextLink.
        /// </summary>
        [JsonPropertyName("columnLinks@odata.nextLink")]
        [JsonConverter(typeof(NextLinkConverter))]
        public string ColumnLinksNextLink { get; set; }
    
        /// <summary>
        /// Gets or sets column positions.
        /// Column order information in a content type.
        /// </summary>
        [JsonPropertyName("columnPositions")]
        public IContentTypeColumnPositionsCollectionWithReferencesPage ColumnPositions { get; set; }

        /// <summary>
        /// Gets or sets columnPositionsNextLink.
        /// </summary>
        [JsonPropertyName("columnPositions@odata.nextLink")]
        [JsonConverter(typeof(NextLinkConverter))]
        public string ColumnPositionsNextLink { get; set; }
    
        /// <summary>
        /// Gets or sets columns.
        /// The collection of column definitions for this contentType.
        /// </summary>
        [JsonPropertyName("columns")]
        public IContentTypeColumnsCollectionPage Columns { get; set; }

        /// <summary>
        /// Gets or sets columnsNextLink.
        /// </summary>
        [JsonPropertyName("columns@odata.nextLink")]
        [JsonConverter(typeof(NextLinkConverter))]
        public string ColumnsNextLink { get; set; }
    
    }
}

