// ------------------------------------------------------------------------------
//  Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
// ------------------------------------------------------------------------------

// **NOTE** This file was generated by a tool and any changes will be overwritten.
// <auto-generated/>

// Template Source: EntityRequest.cs.tt

namespace Microsoft.Graph
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Net.Http;
    using System.Threading;
    using System.Linq.Expressions;

    /// <summary>
    /// The type BitlockerRequest.
    /// </summary>
    public partial class BitlockerRequest : BaseRequest, IBitlockerRequest
    {
        /// <summary>
        /// Constructs a new BitlockerRequest.
        /// </summary>
        /// <param name="requestUrl">The URL for the built request.</param>
        /// <param name="client">The <see cref="IBaseClient"/> for handling requests.</param>
        /// <param name="options">Query and header option name value pairs for the request.</param>
        public BitlockerRequest(
            string requestUrl,
            IBaseClient client,
            IEnumerable<Option> options)
            : base(requestUrl, client, options)
        {
        }

        /// <summary>
        /// Creates the specified Bitlocker using POST.
        /// </summary>
        /// <param name="bitlockerToCreate">The Bitlocker to create.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The created Bitlocker.</returns>
        public async System.Threading.Tasks.Task<Bitlocker> CreateAsync(Bitlocker bitlockerToCreate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.POST;
            var newEntity = await this.SendAsync<Bitlocker>(bitlockerToCreate, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(newEntity);
            return newEntity;
        }

        /// <summary>
        /// Creates the specified Bitlocker using POST and returns a <see cref="GraphResponse{Bitlocker}"/> object.
        /// </summary>
        /// <param name="bitlockerToCreate">The Bitlocker to create.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The <see cref="GraphResponse{Bitlocker}"/> object of the request.</returns>
        public System.Threading.Tasks.Task<GraphResponse<Bitlocker>> CreateResponseAsync(Bitlocker bitlockerToCreate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.POST;
            return this.SendAsyncWithGraphResponse<Bitlocker>(bitlockerToCreate, cancellationToken);
        }

        /// <summary>
        /// Deletes the specified Bitlocker.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task to await.</returns>
        public async System.Threading.Tasks.Task DeleteAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.DELETE;
            await this.SendAsync<Bitlocker>(null, cancellationToken).ConfigureAwait(false);
        }

        /// <summary>
        /// Deletes the specified Bitlocker and returns a <see cref="GraphResponse"/> object.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task of <see cref="GraphResponse"/> to await.</returns>
        public System.Threading.Tasks.Task<GraphResponse> DeleteResponseAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.DELETE;
            return this.SendAsyncWithGraphResponse(null, cancellationToken);
        }

        /// <summary>
        /// Gets the specified Bitlocker.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The Bitlocker.</returns>
        public async System.Threading.Tasks.Task<Bitlocker> GetAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.GET;
            var retrievedEntity = await this.SendAsync<Bitlocker>(null, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(retrievedEntity);
            return retrievedEntity;
        }

        /// <summary>
        /// Gets the specified Bitlocker and returns a <see cref="GraphResponse{Bitlocker}"/> object.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The <see cref="GraphResponse{Bitlocker}"/> object of the request.</returns>
        public System.Threading.Tasks.Task<GraphResponse<Bitlocker>> GetResponseAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.GET;
            return this.SendAsyncWithGraphResponse<Bitlocker>(null, cancellationToken);
        }

        /// <summary>
        /// Updates the specified Bitlocker using PATCH.
        /// </summary>
        /// <param name="bitlockerToUpdate">The Bitlocker to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <exception cref="ClientException">Thrown when an object returned in a response is used for updating an object in Microsoft Graph.</exception>
        /// <returns>The updated Bitlocker.</returns>
        public async System.Threading.Tasks.Task<Bitlocker> UpdateAsync(Bitlocker bitlockerToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PATCH;
            var updatedEntity = await this.SendAsync<Bitlocker>(bitlockerToUpdate, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(updatedEntity);
            return updatedEntity;
        }

        /// <summary>
        /// Updates the specified Bitlocker using PATCH and returns a <see cref="GraphResponse{Bitlocker}"/> object.
        /// </summary>
        /// <param name="bitlockerToUpdate">The Bitlocker to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <exception cref="ClientException">Thrown when an object returned in a response is used for updating an object in Microsoft Graph.</exception>
        /// <returns>The <see cref="GraphResponse{Bitlocker}"/> object of the request.</returns>
        public System.Threading.Tasks.Task<GraphResponse<Bitlocker>> UpdateResponseAsync(Bitlocker bitlockerToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PATCH;
            return this.SendAsyncWithGraphResponse<Bitlocker>(bitlockerToUpdate, cancellationToken);
        }

        /// <summary>
        /// Updates the specified Bitlocker using PUT.
        /// </summary>
        /// <param name="bitlockerToUpdate">The Bitlocker object to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task to await.</returns>
        public async System.Threading.Tasks.Task<Bitlocker> PutAsync(Bitlocker bitlockerToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PUT;
            var updatedEntity = await this.SendAsync<Bitlocker>(bitlockerToUpdate, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(updatedEntity);
            return updatedEntity;
        }

        /// <summary>
        /// Updates the specified Bitlocker using PUT and returns a <see cref="GraphResponse{Bitlocker}"/> object.
        /// </summary>
        /// <param name="bitlockerToUpdate">The Bitlocker object to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task to await of <see cref="GraphResponse{Bitlocker}"/>.</returns>
        public System.Threading.Tasks.Task<GraphResponse<Bitlocker>> PutResponseAsync(Bitlocker bitlockerToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PUT;
            return this.SendAsyncWithGraphResponse<Bitlocker>(bitlockerToUpdate, cancellationToken);
        }

        /// <summary>
        /// Adds the specified expand value to the request.
        /// </summary>
        /// <param name="value">The expand value.</param>
        /// <returns>The request object to send.</returns>
        public IBitlockerRequest Expand(string value)
        {
            this.QueryOptions.Add(new QueryOption("$expand", value));
            return this;
        }

        /// <summary>
        /// Adds the specified expand value to the request.
        /// </summary>
        /// <param name="expandExpression">The expression from which to calculate the expand value.</param>
        /// <returns>The request object to send.</returns>
        public IBitlockerRequest Expand(Expression<Func<Bitlocker, object>> expandExpression)
        {
		    if (expandExpression == null)
            {
                throw new ArgumentNullException(nameof(expandExpression));
            }
            string error;
            string value = ExpressionExtractHelper.ExtractMembers(expandExpression, out error);
            if (value == null)
            {
                throw new ArgumentException(error, nameof(expandExpression));
            }
            else
            {
                this.QueryOptions.Add(new QueryOption("$expand", value));
            }
            return this;
        }

        /// <summary>
        /// Adds the specified select value to the request.
        /// </summary>
        /// <param name="value">The select value.</param>
        /// <returns>The request object to send.</returns>
        public IBitlockerRequest Select(string value)
        {
            this.QueryOptions.Add(new QueryOption("$select", value));
            return this;
        }

        /// <summary>
        /// Adds the specified select value to the request.
        /// </summary>
        /// <param name="selectExpression">The expression from which to calculate the select value.</param>
        /// <returns>The request object to send.</returns>
        public IBitlockerRequest Select(Expression<Func<Bitlocker, object>> selectExpression)
        {
            if (selectExpression == null)
            {
                throw new ArgumentNullException(nameof(selectExpression));
            }
            string error;
            string value = ExpressionExtractHelper.ExtractMembers(selectExpression, out error);
            if (value == null)
            {
                throw new ArgumentException(error, nameof(selectExpression));
            }
            else
            {
                this.QueryOptions.Add(new QueryOption("$select", value));
            }
            return this;
        }

        /// <summary>
        /// Initializes any collection properties after deserialization, like next requests for paging.
        /// </summary>
        /// <param name="bitlockerToInitialize">The <see cref="Bitlocker"/> with the collection properties to initialize.</param>
        private void InitializeCollectionProperties(Bitlocker bitlockerToInitialize)
        {

            if (bitlockerToInitialize != null)
            {
                if (bitlockerToInitialize.RecoveryKeys != null && bitlockerToInitialize.RecoveryKeys.CurrentPage != null)
                {
                    bitlockerToInitialize.RecoveryKeys.InitializeNextPageRequest(this.Client, bitlockerToInitialize.RecoveryKeysNextLink);
                    // Copy the additional data collection to the page itself so that information is not lost
                    bitlockerToInitialize.RecoveryKeys.AdditionalData = bitlockerToInitialize.AdditionalData;
                }

            }


        }
    }
}
