// ------------------------------------------------------------------------------
//  Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
// ------------------------------------------------------------------------------

// **NOTE** This file was generated by a tool and any changes will be overwritten.
// <auto-generated/>

// Template Source: EntityRequest.cs.tt

namespace Microsoft.Graph
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Net.Http;
    using System.Threading;
    using System.Linq.Expressions;

    /// <summary>
    /// The type ListRequest.
    /// </summary>
    public partial class ListRequest : BaseRequest, IListRequest
    {
        /// <summary>
        /// Constructs a new ListRequest.
        /// </summary>
        /// <param name="requestUrl">The URL for the built request.</param>
        /// <param name="client">The <see cref="IBaseClient"/> for handling requests.</param>
        /// <param name="options">Query and header option name value pairs for the request.</param>
        public ListRequest(
            string requestUrl,
            IBaseClient client,
            IEnumerable<Option> options)
            : base(requestUrl, client, options)
        {
        }

        /// <summary>
        /// Creates the specified List using POST.
        /// </summary>
        /// <param name="listToCreate">The List to create.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The created List.</returns>
        public async System.Threading.Tasks.Task<List> CreateAsync(List listToCreate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.POST;
            var newEntity = await this.SendAsync<List>(listToCreate, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(newEntity);
            return newEntity;
        }

        /// <summary>
        /// Creates the specified List using POST and returns a <see cref="GraphResponse{List}"/> object.
        /// </summary>
        /// <param name="listToCreate">The List to create.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The <see cref="GraphResponse{List}"/> object of the request.</returns>
        public System.Threading.Tasks.Task<GraphResponse<List>> CreateResponseAsync(List listToCreate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.POST;
            return this.SendAsyncWithGraphResponse<List>(listToCreate, cancellationToken);
        }

        /// <summary>
        /// Deletes the specified List.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task to await.</returns>
        public async System.Threading.Tasks.Task DeleteAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.DELETE;
            await this.SendAsync<List>(null, cancellationToken).ConfigureAwait(false);
        }

        /// <summary>
        /// Deletes the specified List and returns a <see cref="GraphResponse"/> object.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task of <see cref="GraphResponse"/> to await.</returns>
        public System.Threading.Tasks.Task<GraphResponse> DeleteResponseAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.DELETE;
            return this.SendAsyncWithGraphResponse(null, cancellationToken);
        }

        /// <summary>
        /// Gets the specified List.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The List.</returns>
        public async System.Threading.Tasks.Task<List> GetAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.GET;
            var retrievedEntity = await this.SendAsync<List>(null, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(retrievedEntity);
            return retrievedEntity;
        }

        /// <summary>
        /// Gets the specified List and returns a <see cref="GraphResponse{List}"/> object.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The <see cref="GraphResponse{List}"/> object of the request.</returns>
        public System.Threading.Tasks.Task<GraphResponse<List>> GetResponseAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.GET;
            return this.SendAsyncWithGraphResponse<List>(null, cancellationToken);
        }

        /// <summary>
        /// Updates the specified List using PATCH.
        /// </summary>
        /// <param name="listToUpdate">The List to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <exception cref="ClientException">Thrown when an object returned in a response is used for updating an object in Microsoft Graph.</exception>
        /// <returns>The updated List.</returns>
        public async System.Threading.Tasks.Task<List> UpdateAsync(List listToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PATCH;
            var updatedEntity = await this.SendAsync<List>(listToUpdate, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(updatedEntity);
            return updatedEntity;
        }

        /// <summary>
        /// Updates the specified List using PATCH and returns a <see cref="GraphResponse{List}"/> object.
        /// </summary>
        /// <param name="listToUpdate">The List to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <exception cref="ClientException">Thrown when an object returned in a response is used for updating an object in Microsoft Graph.</exception>
        /// <returns>The <see cref="GraphResponse{List}"/> object of the request.</returns>
        public System.Threading.Tasks.Task<GraphResponse<List>> UpdateResponseAsync(List listToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PATCH;
            return this.SendAsyncWithGraphResponse<List>(listToUpdate, cancellationToken);
        }

        /// <summary>
        /// Updates the specified List using PUT.
        /// </summary>
        /// <param name="listToUpdate">The List object to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task to await.</returns>
        public async System.Threading.Tasks.Task<List> PutAsync(List listToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PUT;
            var updatedEntity = await this.SendAsync<List>(listToUpdate, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(updatedEntity);
            return updatedEntity;
        }

        /// <summary>
        /// Updates the specified List using PUT and returns a <see cref="GraphResponse{List}"/> object.
        /// </summary>
        /// <param name="listToUpdate">The List object to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task to await of <see cref="GraphResponse{List}"/>.</returns>
        public System.Threading.Tasks.Task<GraphResponse<List>> PutResponseAsync(List listToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PUT;
            return this.SendAsyncWithGraphResponse<List>(listToUpdate, cancellationToken);
        }

        /// <summary>
        /// Adds the specified expand value to the request.
        /// </summary>
        /// <param name="value">The expand value.</param>
        /// <returns>The request object to send.</returns>
        public IListRequest Expand(string value)
        {
            this.QueryOptions.Add(new QueryOption("$expand", value));
            return this;
        }

        /// <summary>
        /// Adds the specified expand value to the request.
        /// </summary>
        /// <param name="expandExpression">The expression from which to calculate the expand value.</param>
        /// <returns>The request object to send.</returns>
        public IListRequest Expand(Expression<Func<List, object>> expandExpression)
        {
		    if (expandExpression == null)
            {
                throw new ArgumentNullException(nameof(expandExpression));
            }
            string error;
            string value = ExpressionExtractHelper.ExtractMembers(expandExpression, out error);
            if (value == null)
            {
                throw new ArgumentException(error, nameof(expandExpression));
            }
            else
            {
                this.QueryOptions.Add(new QueryOption("$expand", value));
            }
            return this;
        }

        /// <summary>
        /// Adds the specified select value to the request.
        /// </summary>
        /// <param name="value">The select value.</param>
        /// <returns>The request object to send.</returns>
        public IListRequest Select(string value)
        {
            this.QueryOptions.Add(new QueryOption("$select", value));
            return this;
        }

        /// <summary>
        /// Adds the specified select value to the request.
        /// </summary>
        /// <param name="selectExpression">The expression from which to calculate the select value.</param>
        /// <returns>The request object to send.</returns>
        public IListRequest Select(Expression<Func<List, object>> selectExpression)
        {
            if (selectExpression == null)
            {
                throw new ArgumentNullException(nameof(selectExpression));
            }
            string error;
            string value = ExpressionExtractHelper.ExtractMembers(selectExpression, out error);
            if (value == null)
            {
                throw new ArgumentException(error, nameof(selectExpression));
            }
            else
            {
                this.QueryOptions.Add(new QueryOption("$select", value));
            }
            return this;
        }

        /// <summary>
        /// Initializes any collection properties after deserialization, like next requests for paging.
        /// </summary>
        /// <param name="listToInitialize">The <see cref="List"/> with the collection properties to initialize.</param>
        private void InitializeCollectionProperties(List listToInitialize)
        {

            if (listToInitialize != null)
            {
                if (listToInitialize.Columns != null && listToInitialize.Columns.CurrentPage != null)
                {
                    listToInitialize.Columns.InitializeNextPageRequest(this.Client, listToInitialize.ColumnsNextLink);
                    // Copy the additional data collection to the page itself so that information is not lost
                    listToInitialize.Columns.AdditionalData = listToInitialize.AdditionalData;
                }
                if (listToInitialize.ContentTypes != null && listToInitialize.ContentTypes.CurrentPage != null)
                {
                    listToInitialize.ContentTypes.InitializeNextPageRequest(this.Client, listToInitialize.ContentTypesNextLink);
                    // Copy the additional data collection to the page itself so that information is not lost
                    listToInitialize.ContentTypes.AdditionalData = listToInitialize.AdditionalData;
                }
                if (listToInitialize.Items != null && listToInitialize.Items.CurrentPage != null)
                {
                    listToInitialize.Items.InitializeNextPageRequest(this.Client, listToInitialize.ItemsNextLink);
                    // Copy the additional data collection to the page itself so that information is not lost
                    listToInitialize.Items.AdditionalData = listToInitialize.AdditionalData;
                }
                if (listToInitialize.Subscriptions != null && listToInitialize.Subscriptions.CurrentPage != null)
                {
                    listToInitialize.Subscriptions.InitializeNextPageRequest(this.Client, listToInitialize.SubscriptionsNextLink);
                    // Copy the additional data collection to the page itself so that information is not lost
                    listToInitialize.Subscriptions.AdditionalData = listToInitialize.AdditionalData;
                }

            }


        }
    }
}
