// ------------------------------------------------------------------------------
//  Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
// ------------------------------------------------------------------------------

// **NOTE** This file was generated by a tool and any changes will be overwritten.
// <auto-generated/>

// Template Source: EntityRequest.cs.tt

namespace Microsoft.Graph
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Net.Http;
    using System.Threading;
    using System.Linq.Expressions;

    /// <summary>
    /// The type PrinterRequest.
    /// </summary>
    public partial class PrinterRequest : BaseRequest, IPrinterRequest
    {
        /// <summary>
        /// Constructs a new PrinterRequest.
        /// </summary>
        /// <param name="requestUrl">The URL for the built request.</param>
        /// <param name="client">The <see cref="IBaseClient"/> for handling requests.</param>
        /// <param name="options">Query and header option name value pairs for the request.</param>
        public PrinterRequest(
            string requestUrl,
            IBaseClient client,
            IEnumerable<Option> options)
            : base(requestUrl, client, options)
        {
        }

        /// <summary>
        /// Creates the specified Printer using POST.
        /// </summary>
        /// <param name="printerToCreate">The Printer to create.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The created Printer.</returns>
        public async System.Threading.Tasks.Task<Printer> CreateAsync(Printer printerToCreate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.POST;
            var newEntity = await this.SendAsync<Printer>(printerToCreate, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(newEntity);
            return newEntity;
        }

        /// <summary>
        /// Creates the specified Printer using POST and returns a <see cref="GraphResponse{Printer}"/> object.
        /// </summary>
        /// <param name="printerToCreate">The Printer to create.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The <see cref="GraphResponse{Printer}"/> object of the request.</returns>
        public System.Threading.Tasks.Task<GraphResponse<Printer>> CreateResponseAsync(Printer printerToCreate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.POST;
            return this.SendAsyncWithGraphResponse<Printer>(printerToCreate, cancellationToken);
        }

        /// <summary>
        /// Deletes the specified Printer.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task to await.</returns>
        public async System.Threading.Tasks.Task DeleteAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.DELETE;
            await this.SendAsync<Printer>(null, cancellationToken).ConfigureAwait(false);
        }

        /// <summary>
        /// Deletes the specified Printer and returns a <see cref="GraphResponse"/> object.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task of <see cref="GraphResponse"/> to await.</returns>
        public System.Threading.Tasks.Task<GraphResponse> DeleteResponseAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.DELETE;
            return this.SendAsyncWithGraphResponse(null, cancellationToken);
        }

        /// <summary>
        /// Gets the specified Printer.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The Printer.</returns>
        public async System.Threading.Tasks.Task<Printer> GetAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.GET;
            var retrievedEntity = await this.SendAsync<Printer>(null, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(retrievedEntity);
            return retrievedEntity;
        }

        /// <summary>
        /// Gets the specified Printer and returns a <see cref="GraphResponse{Printer}"/> object.
        /// </summary>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The <see cref="GraphResponse{Printer}"/> object of the request.</returns>
        public System.Threading.Tasks.Task<GraphResponse<Printer>> GetResponseAsync(CancellationToken cancellationToken = default)
        {
            this.Method = HttpMethods.GET;
            return this.SendAsyncWithGraphResponse<Printer>(null, cancellationToken);
        }

        /// <summary>
        /// Updates the specified Printer using PATCH.
        /// </summary>
        /// <param name="printerToUpdate">The Printer to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <exception cref="ClientException">Thrown when an object returned in a response is used for updating an object in Microsoft Graph.</exception>
        /// <returns>The updated Printer.</returns>
        public async System.Threading.Tasks.Task<Printer> UpdateAsync(Printer printerToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PATCH;
            var updatedEntity = await this.SendAsync<Printer>(printerToUpdate, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(updatedEntity);
            return updatedEntity;
        }

        /// <summary>
        /// Updates the specified Printer using PATCH and returns a <see cref="GraphResponse{Printer}"/> object.
        /// </summary>
        /// <param name="printerToUpdate">The Printer to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <exception cref="ClientException">Thrown when an object returned in a response is used for updating an object in Microsoft Graph.</exception>
        /// <returns>The <see cref="GraphResponse{Printer}"/> object of the request.</returns>
        public System.Threading.Tasks.Task<GraphResponse<Printer>> UpdateResponseAsync(Printer printerToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PATCH;
            return this.SendAsyncWithGraphResponse<Printer>(printerToUpdate, cancellationToken);
        }

        /// <summary>
        /// Updates the specified Printer using PUT.
        /// </summary>
        /// <param name="printerToUpdate">The Printer object to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task to await.</returns>
        public async System.Threading.Tasks.Task<Printer> PutAsync(Printer printerToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PUT;
            var updatedEntity = await this.SendAsync<Printer>(printerToUpdate, cancellationToken).ConfigureAwait(false);
            this.InitializeCollectionProperties(updatedEntity);
            return updatedEntity;
        }

        /// <summary>
        /// Updates the specified Printer using PUT and returns a <see cref="GraphResponse{Printer}"/> object.
        /// </summary>
        /// <param name="printerToUpdate">The Printer object to update.</param>
        /// <param name="cancellationToken">The <see cref="CancellationToken"/> for the request.</param>
        /// <returns>The task to await of <see cref="GraphResponse{Printer}"/>.</returns>
        public System.Threading.Tasks.Task<GraphResponse<Printer>> PutResponseAsync(Printer printerToUpdate, CancellationToken cancellationToken = default)
        {
            this.ContentType = CoreConstants.MimeTypeNames.Application.Json;
            this.Method = HttpMethods.PUT;
            return this.SendAsyncWithGraphResponse<Printer>(printerToUpdate, cancellationToken);
        }

        /// <summary>
        /// Adds the specified expand value to the request.
        /// </summary>
        /// <param name="value">The expand value.</param>
        /// <returns>The request object to send.</returns>
        public IPrinterRequest Expand(string value)
        {
            this.QueryOptions.Add(new QueryOption("$expand", value));
            return this;
        }

        /// <summary>
        /// Adds the specified expand value to the request.
        /// </summary>
        /// <param name="expandExpression">The expression from which to calculate the expand value.</param>
        /// <returns>The request object to send.</returns>
        public IPrinterRequest Expand(Expression<Func<Printer, object>> expandExpression)
        {
		    if (expandExpression == null)
            {
                throw new ArgumentNullException(nameof(expandExpression));
            }
            string error;
            string value = ExpressionExtractHelper.ExtractMembers(expandExpression, out error);
            if (value == null)
            {
                throw new ArgumentException(error, nameof(expandExpression));
            }
            else
            {
                this.QueryOptions.Add(new QueryOption("$expand", value));
            }
            return this;
        }

        /// <summary>
        /// Adds the specified select value to the request.
        /// </summary>
        /// <param name="value">The select value.</param>
        /// <returns>The request object to send.</returns>
        public IPrinterRequest Select(string value)
        {
            this.QueryOptions.Add(new QueryOption("$select", value));
            return this;
        }

        /// <summary>
        /// Adds the specified select value to the request.
        /// </summary>
        /// <param name="selectExpression">The expression from which to calculate the select value.</param>
        /// <returns>The request object to send.</returns>
        public IPrinterRequest Select(Expression<Func<Printer, object>> selectExpression)
        {
            if (selectExpression == null)
            {
                throw new ArgumentNullException(nameof(selectExpression));
            }
            string error;
            string value = ExpressionExtractHelper.ExtractMembers(selectExpression, out error);
            if (value == null)
            {
                throw new ArgumentException(error, nameof(selectExpression));
            }
            else
            {
                this.QueryOptions.Add(new QueryOption("$select", value));
            }
            return this;
        }

        /// <summary>
        /// Initializes any collection properties after deserialization, like next requests for paging.
        /// </summary>
        /// <param name="printerToInitialize">The <see cref="Printer"/> with the collection properties to initialize.</param>
        private void InitializeCollectionProperties(Printer printerToInitialize)
        {

            if (printerToInitialize != null)
            {
                if (printerToInitialize.Connectors != null && printerToInitialize.Connectors.CurrentPage != null)
                {
                    printerToInitialize.Connectors.InitializeNextPageRequest(this.Client, printerToInitialize.ConnectorsNextLink);
                    // Copy the additional data collection to the page itself so that information is not lost
                    printerToInitialize.Connectors.AdditionalData = printerToInitialize.AdditionalData;
                }
                if (printerToInitialize.Shares != null && printerToInitialize.Shares.CurrentPage != null)
                {
                    printerToInitialize.Shares.InitializeNextPageRequest(this.Client, printerToInitialize.SharesNextLink);
                    // Copy the additional data collection to the page itself so that information is not lost
                    printerToInitialize.Shares.AdditionalData = printerToInitialize.AdditionalData;
                }
                if (printerToInitialize.TaskTriggers != null && printerToInitialize.TaskTriggers.CurrentPage != null)
                {
                    printerToInitialize.TaskTriggers.InitializeNextPageRequest(this.Client, printerToInitialize.TaskTriggersNextLink);
                    // Copy the additional data collection to the page itself so that information is not lost
                    printerToInitialize.TaskTriggers.AdditionalData = printerToInitialize.AdditionalData;
                }

            }


        }
    }
}
